@@ -297,7 +297,7 @@ static const SECTION union { const guint8 data[2303]; const double alignment; vo
   0x00, 0x28, 0x75, 0x75, 0x61, 0x79, 0x29
 } };
 
-static GStaticResource static_resource = { gpm_resource_data.data, sizeof (gpm_resource_data.data) };
+static GStaticResource static_resource = { gpm_resource_data.data, sizeof (gpm_resource_data.data), NULL, NULL, NULL };
 extern GResource *gpm_get_resource (void);
 GResource *gpm_get_resource (void)
 {
